#include "HomeSpan.h" 

#define waterSensorPin 17
#define relaisPin 13

bool empty = false;
bool lastState = false;


struct Valve : Service::Valve { 

  SpanCharacteristic *name =new Characteristic::ConfiguredName("Smart Garden");
  SpanCharacteristic *active =new Characteristic::Active(0);
  SpanCharacteristic *inUse =new Characteristic::InUse(0);
  SpanCharacteristic *setDuration =new Characteristic::SetDuration(60);
  SpanCharacteristic *remainingDuration =new Characteristic::RemainingDuration(0);
  SpanCharacteristic *enabled =new Characteristic::IsConfigured(1);

  Valve() : Service::Valve() {
    new Characteristic::ValveType(1);
    enabled->addPerms(PW);
  }

  boolean update(){            

    if(enabled->updated()){
      if(enabled->getNewVal()){
        Serial.println("Smart Garden Active");
      } else {   
         Serial.println("Smart Garden Inactive");     
        if(active->getVal()){
          active->setVal(0);
          inUse->setVal(0);
          remainingDuration->setVal(0);
          digitalWrite(relaisPin, LOW);    
        }
      }
    }

    if(active->updated()){
      if(active->getNewVal() && !empty){
        Serial.println("Valve Open");
        inUse->setVal(1);
        remainingDuration->setVal(setDuration->getVal()); 
        digitalWrite(relaisPin, HIGH);
      } else {
        Serial.println("Valve Closed");
        inUse->setVal(0);
        active->setVal(0);
        remainingDuration->setVal(0); 
        digitalWrite(relaisPin, LOW);
      }
    }
    return(true);
  }


  void loop() override {

    empty = !digitalRead(waterSensorPin);

    if(empty == true && lastState == false){
      Serial.println("Waterlevel Empty");
      Serial.println("Valve Closed");
      lastState = true;
      active->setVal(0);
      inUse->setVal(0);
      remainingDuration->setVal(0);
      digitalWrite(relaisPin, LOW);
    }else if(empty == false && lastState == true){
      Serial.println("Waterlevel Filled");
      lastState = false;
    }


    if(active->getVal()){
      int remainingTime=setDuration->getVal()-active->timeVal()/1000;
         
      if(remainingTime<=0){
        Serial.println("Timer Over");
        active->setVal(0);
        inUse->setVal(0);
        remainingDuration->setVal(0);
        digitalWrite(relaisPin, LOW);
      } else

      if(remainingTime<remainingDuration->getVal()){
        remainingDuration->setVal(remainingTime,false);
      }
    }
  }
};


void setup() {     

  Serial.begin(115200); 

  pinMode(waterSensorPin, INPUT);
  pinMode(relaisPin, OUTPUT);
  digitalWrite(relaisPin, LOW);

  homeSpan.begin(Category::Sprinklers,"Smart Garden");  
  
  new SpanAccessory();
    new Service::AccessoryInformation();
      new Characteristic::Name("Smart Garden");                   
      new Characteristic::Manufacturer("SRoessger");             
      new Characteristic::SerialNumber("0001");              
      new Characteristic::Model("Smart Garden - Indoor");                    
      new Characteristic::FirmwareRevision(HOMESPAN_VERSION);   
      new Characteristic::Identify();                           
  
    new Service::HAPProtocolInformation();
      new Characteristic::Version("1.1.0"); 

  new Valve();

} 

void loop(){

  homeSpan.poll();

}